﻿using Dalamud.Game.Command;
using Dalamud.IoC;
using Dalamud.Plugin;
using System;
using System.Net.Http;
using System.Text;
using Newtonsoft.Json;
using Dalamud.Interface.Windowing;
using Dalamud.Plugin.Services;
using DalamudShock.Windows;
// ReSharper disable RedundantAnonymousTypePropertyName


namespace DalamudShock
{
    public sealed class Plugin : IDalamudPlugin
    {
        [PluginService] internal static IDalamudPluginInterface PluginInterface { get; private set; } = null!;
        [PluginService] internal static IClientState ClientState { get; private set; } = null!;
        [PluginService] internal static IFramework Framework { get; private set; } = null!;
        [PluginService] internal static ICommandManager CommandManager { get; private set; } = null!;
        [PluginService] internal static IPluginLog PluginLog { get; private set; } = null!;


        private const string CommandName = "/zap";
        public Configuration Configuration { get; init; }
        public readonly WindowSystem WindowSystem = new("DalamudShock");
        private ConfigWindow ConfigWindow { get; init; }
        private MainWindow MainWindow { get; init; }
        private uint lastHealth;
        private DateTime lastShockTime = DateTime.MinValue;

        public Plugin()
        {
            Configuration = PluginInterface.GetPluginConfig() as Configuration ?? new Configuration();
            
            ConfigWindow = new ConfigWindow(this);
            MainWindow = new MainWindow(this);

            WindowSystem.AddWindow(ConfigWindow);
            WindowSystem.AddWindow(MainWindow);

            CommandManager.AddHandler(CommandName, new CommandInfo(OnCommand)
            {
                HelpMessage = "A shockingly empty description :3"
            });

            PluginInterface.UiBuilder.Draw += DrawUI;
            PluginInterface.UiBuilder.OpenConfigUi += ToggleConfigUI;
            PluginInterface.UiBuilder.OpenMainUi += ToggleMainUI;

            Framework.Update += OnFrameworkUpdate;
        }

        public void Dispose()
        {
            Framework.Update -= OnFrameworkUpdate;
            WindowSystem.RemoveAllWindows();
            ConfigWindow.Dispose();
            MainWindow.Dispose();
            CommandManager.RemoveHandler(CommandName);
        }

        private void OnCommand(string command, string args)
        {
            ToggleMainUI();
        }

        private void DrawUI() => WindowSystem.Draw();

        public void ToggleConfigUI() => ConfigWindow.Toggle();
        public void ToggleMainUI() => MainWindow.Toggle();

        private void OnFrameworkUpdate(IFramework framework)
        {
            var player = ClientState.LocalPlayer;

            if (player == null) return; 
            var currentHealth = player.CurrentHp;
            
            if (currentHealth == lastHealth) return;
            lastHealth = currentHealth;

            if (currentHealth != lastHealth)
            {
                lastHealth = currentHealth;

                if (Configuration.ZapOnDamage)
                {
                    if ((DateTime.Now - lastShockTime).TotalSeconds >= 15)
                    {
                        lastShockTime = DateTime.Now;

                        if (Configuration.UsePishock)
                        {
                            SendPiShockApiRequest();
                        }
                        else
                        {
                            SendOscApiRequest();
                        }
                    }
                }
                else
                {
                    if (currentHealth < player.MaxHp * 0.5)
                    {
                        if ((DateTime.Now - lastShockTime).TotalSeconds >= 15)
                        {
                            PluginLog.Information($"Player health changed to {currentHealth}, time conditions met.");
                            lastShockTime = DateTime.Now;

                            if (Configuration.UsePishock)
                            {
                                PluginLog.Information("Using PiShock, triggering PiShock API request.");
                                SendPiShockApiRequest();
                            }
                            else
                            {
                                PluginLog.Information(
                                    "Using default shock API, triggering standard shock API request.");
                                SendOscApiRequest();
                            }
                        }
                    }
                }
            }
        }

        private async void SendOscApiRequest()
        {
            if (string.IsNullOrEmpty(Configuration.openshockApiKey) || string.IsNullOrEmpty(Configuration.OpenshockShockerID))
            {
                PluginLog.Error("OpenShock API Key or Shocker ID is not configured.");
                return;
            }

            var isVibrationAllowed = Configuration.IsVibrationAllowed;

            var random = new Random();
            var intensity = random.Next(0, Configuration.MaxValue);
            var duration = random.Next(1000, Configuration.MaxDuration * 1000);

            // Calculate the type based on weighted odds
            var type = "shock"; 
            if (isVibrationAllowed)
            {
                var chance = random.Next(1, 11); 
                if (chance > 8)                  
                {
                    type = "vibrate";
                }
            }

            var requestData = new[]
            {
                new
                {
                    id = Configuration.OpenshockShockerID,
                    type = type,
                    intensity = intensity,
                    duration = duration
                }
            };



            var jsonContent = JsonConvert.SerializeObject(requestData);
            var content = new StringContent(jsonContent, Encoding.UTF8, "application/json");

            using var client = new HttpClient();
            client.DefaultRequestHeaders.Add("OpenShockToken", Configuration.openshockApiKey);
            client.DefaultRequestHeaders.Add("User-Agent", 
                                             "Dalamud/0.2 (Windows; U; Windows NT 10.4; x64) AppleWebKit/602.39 (KHTML, like Gecko)");

            try
            {
                var response = await client.PostAsync("https://api.shocklink.net/1/shockers/control", content);

                if (response.IsSuccessStatusCode)
                {
                    PluginLog.Information("{0} sent successfully with intensity {1} and duration {2} ms.", type, intensity, duration);
                }
                else
                {
                    var responseBody = await response.Content.ReadAsStringAsync();
                    PluginLog.Error($"Shock API request failed: {responseBody}");
                }
            }
            catch (Exception ex)
            {
                PluginLog.Error($"Error occurred while sending shock: {ex.Message}");
            }
        }
        
        private async void SendPiShockApiRequest()
        {
            if (string.IsNullOrEmpty(Configuration.openshockApiKey) || string.IsNullOrEmpty(Configuration.OpenshockShockerID))
            {
                PluginLog.Error("Pishock API Key or ShareLink is not configured.");
                return;
            }

            var isVibrationAllowed = Configuration.IsVibrationAllowed;

            var random = new Random();
            var intensity = random.Next(0, Configuration.MaxValue);
            var duration = random.Next(1000, Configuration.MaxDuration);

            var type = "0"; 
            if (isVibrationAllowed)
            {
                var chance = random.Next(1, 11); 
                if (chance > 8)                  
                {
                    type = "1";
                }
            }

            var requestData = new[]
            {
                new
                {
                    Username = Configuration.PiShockUser,
                    Name = "DalamudShock",
                    Code = Configuration.OpenshockShockerID,
                    Intensity = intensity,
                    Duration = duration,
                    Op= type
                }
            };



            var jsonContent = JsonConvert.SerializeObject(requestData);
            var content = new StringContent(jsonContent, Encoding.UTF8, "application/json");

            using var client = new HttpClient();
            client.DefaultRequestHeaders.Add("User-Agent", 
                                             "Dalamud/0.2 (Windows; U; Windows NT 10.4; x64) AppleWebKit/602.39 (KHTML, like Gecko)");

            try
            {
                var response = await client.PostAsync("https://do.pishock.com/api/apioperate", content);

                if (response.IsSuccessStatusCode)
                {
                    PluginLog.Information("{0} sent successfully with intensity {1} and duration {2} ms.", type, intensity, duration);
                }
                else
                {
                    var responseBody = await response.Content.ReadAsStringAsync();
                    PluginLog.Error($"Shock API request failed: {responseBody}");
                }
            }
            catch (Exception ex)
            {
                PluginLog.Error($"Error occurred while sending shock: {ex.Message}");
            }
        }

    }
}

